#include <stdio.h> 
#include "M0518.h" 
//#include "cec_ctrl.h" 
 
#define  PLL_CLOCK  48000000 
 
/* Definition for CEC */ 
// Bit Timing 
#define  CEC_START_BIT_L  37  // 37 x 100us = 3.7ms 
#define  CEC_START_BIT_H   8  //  8 x 100us = 0.8ms 
#define  CEC_BIT_1_L    6  //  6 x 100us = 0.6ms 
#define  CEC_BIT_1_H   18   // 18 x 100us = 1.8ms 
#define  CEC_BIT_0_L   15  // 15 x 100us = 1.5ms 
#define  CEC_BIT_0_H    9  //  9 x 100us = 0.9ms 
#define  CEC_MAX_RETRY    5  // Retry  
#define  CEC_FREE_TIME_PI  7*24  // Present Initator wants to send another frame immediately after its previous frame 
#define  CEC_FREE_TIME_NI  5*24  // New Initator wants to send a frame 
#define  CEC_FREE_TIME_RS  3*24  // Previous attempt to send frame unsuccessful 
// Destination of Header Block 
#define  CEC_HEAD_DEST   0x00  // Header Block Destination  
// Tolerance  
#define  Percent      0.05  // 5 %  
#define  Percent0     0.13  // 13 % 
#define  Percent1     0.33  // 33 % 
// Start Bit range 
#define  STB_MAX      (CEC_START_BIT_L * 4800 * (1 + Percent))   
#define  STB_MIN      (CEC_START_BIT_L * 4800 * (1 - Percent))   
// Bit = 0 range 
#define  BIT_0_MAX     (CEC_BIT_0_L * 4800 * (1 + Percent0))  
#define  BIT_0_MIN    (CEC_BIT_0_L * 4800 * (1 - Percent0))  
// Bit = 1 range 
#define  BIT_1_MAX     (CEC_BIT_1_L * 4800 * (1 + Percent1))  
#define  BIT_1_MIN    (CEC_BIT_1_L * 4800 * (1 - Percent1))  
 
/* Definition for CEC Commands */ 
#define Give_Physical_Address  0x83 
#define  Report_Physical_Address 0x84 
#define Give_Device_Power_Status 0x8f 
#define Report_Power_Status  0x90 
#define Active_Source    0x82 
#define Inactive_Source   0x9d 
#define Image_View_On    0x04 
#define Text_View_On    0x0d 
#define Get_CEC_Version   0x9f 
#define  CEC_Version    0x9e 
#define Give_Device_Vendor_ID  0x8c 
#define Device_Vendor_ID   0x87 
#define Manu_Status    0x8e 
#define Feature_Abort    0x00 
#define Standby     0x36    
 
/* Declare variables */ 
volatile uint32_t  Timer0_int_cnt = 0;  // Timer0 interrupt counter 
 
volatile uint8_t   CEC_State = 0;   // CEC State 
volatile uint8_t   CEC_STB_Ready = 0;  // CEC STart Bit is Ready 
 
volatile uint8_t   CEC_HEAD = 0;    // Received Header  
volatile uint8_t   CEC_DATA = 0;    // Received Data 
volatile  int8_t   CEC_EOM_H = -1;   // End of Message for Header Block 
volatile  int8_t   CEC_EOM_D = -1;   // End of Message for Data Block 
volatile  int8_t   CEC_ACK_H = -1;   // ACK for Header Block 
volatile  int8_t   CEC_ACK_D = -1;   // ACK for Data Block 
 
volatile uint8_t   Initiator_Device = 0;  // Initiator Device  
 
volatile uint8_t   Tx_Retry_Cnt = 0;   // Tx Retry Count  
volatile uint8_t   CEC_Head_Dest_flag = 0; // CEC Header Dest. is correct 
volatile uint8_t   CEC_Broadcast_flag = 0; // 1=CEC Broadcast Command, 0=None 
volatile uint8_t   Tx_flag = 0;      // 1=Tx, 0=Rx 
volatile uint8_t   Tx_NACK_H = 0;   // Tx receive NACK for Header Block 
volatile uint8_t   Tx_NACK_D = 0;   // Tx receive NACK for Data Block 
volatile uint8_t   Rx_CEC_Data_flag = 0;  // 1=received CEC data, 0=None 
volatile uint8_t   Rx_CEC_Data_Len = 0;   // Received data length 
volatile uint8_t  RX_DATA[17] = {0};  // Received Data 
volatile uint8_t   Key_flag = 0;      // 1=key interrupt flag, 0=None 
volatile uint8_t   TMR1_flag = 0;   // 1=TMR1 ISR, 0=No TMR1_ISR 
 
uint8_t REPORT_PHYSICAL_ADDRESS[5] = {0x0f,Report_Physical_Address,0x00,0x00, 0x00};  
uint8_t DEVICE_VENDOR_ID[5] = {0x0f,Device_Vendor_ID,0x00,0x00,0x00};  
uint8_t CEC_VERSION[3] = {0x00,CEC_Version,0x04}; // 0x04:1.3a 
uint8_t REPORT_POWER_STATUS[3] = {0x00,Report_Power_Status,0x00};// 0x00:On, 0x01:OFF 
uint8_t STANDBY[2] = {0x00,Standby}; // 0x04: DVD Player/Chromecast, 0x08:SONY PS3     

void SYS_Init(void) 
{ 
/*----------------------------------------------------------------------------------*/ 
/* Init System Clock                                                                */ 
/*----------------------------------------------------------------------------------*/ 
 /* Unlock protected registers */ 
 SYS_UnlockReg(); 
 
 /* Enable external 12MHz XTAL */ 
 CLK->PWRCON |= CLK_PWRCON_XTL12M_EN_Msk; 
  
 /* Enable internal 10kHz RC */ 
 CLK->PWRCON |= CLK_PWRCON_OSC10K_EN_Msk; 
 
 /* PLL_OUT = 48 MHz */ 
 CLK->PLLCON = 0xCA6E; 
 
 /* Waiting for clock ready */ 
 CLK_WaitClockReady(CLK_CLKSTATUS_PLL_STB_Msk | CLK_CLKSTATUS_OSC10K_STB_Msk | CLK_CLKSTATUS_XTL12M_STB_Msk); 
 
 /* Switch HCLK clock source to PLL */ 
 CLK->CLKSEL0 = CLK_CLKSEL0_HCLK_S_PLL; 
 
 /* Enable IP clock */         
 CLK->APBCLK = CLK_APBCLK_UART0_EN_Msk | CLK_APBCLK_TMR0_EN_Msk | CLK_APBCLK_TMR1_EN_Msk; 
 
 /* IP clock source */ 
 CLK->CLKSEL1 = CLK_CLKSEL1_UART_S_PLL | CLK_CLKSEL1_TMR0_S_HXT | CLK_CLKSEL1_TMR1_S_HCLK; 
 
 /* Update System Core Clock */ 
 /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */ 
 PllClock        = PLL_CLOCK;            // PLL 
 SystemCoreClock = PLL_CLOCK / 1;        // HCLK 
 CyclesPerUs     = PLL_CLOCK / 1000000;  // For SYS_SysTickDelay() 
 
/*----------------------------------------------------------------------------------*/ 
/* Init I/O Multi-function                                                          */ 
/*----------------------------------------------------------------------------------*/ 
 /* Set PB multi-function pins for UART0 RXD, TXD */ 
   SYS->GPB_MFP = SYS_GPB_MFP_PB0_UART0_RXD | SYS_GPB_MFP_PB1_UART0_TXD; 
 
 /* Set PE multi-function pins for TM1_EXT on PE.5 */ 
   SYS->GPE_MFP |= SYS_GPE_MFP_PE5_TM1_EXT; 
   SYS->ALT_MFP |= SYS_ALT_MFP_PE5_TM1_EXT; 
 SYS->ALT_MFP2 |= SYS_ALT_MFP2_PE5_TM1_EXT; 
   
 /* Lock protected registers */ 
 SYS_LockReg(); 
} 

void UART0_Init(void) 
{ 
/*----------------------------------------------------------------------------------*/ 
/* Init UART                                                                        */ 
/*----------------------------------------------------------------------------------*/ 
  /* Reset IP */ 
  SYS_ResetModule(UART0_RST); 
 
  /* Configure UART0 and set UART0 Baudrate */ 
  UART_Open(UART0, 115200); 
} 

//******************************* 
// Initial Timer0 interrupt (100us) 
// 
// Use 12 MHz crystal as Timer0 clock source 
// Use mode 1 (periodic mode) 
// Timer0 is 24-bit resolution 
//******************************* 
void Init_Timer0(void) 
{ 
  /* Reset Timer0 counter, prescale and disable Timer0 (CEN = 0) */ 
  TIMER0->TCSR |= TIMER_TCSR_CRST_Msk; 
 
  /* Select Operation mode */  
  TIMER0->TCSR &= ~TIMER_TCSR_MODE_Msk; 
  TIMER0->TCSR |= TIMER_PERIODIC_MODE;  // Select periodic mode for operation mode and enable TDR function 
 
  /* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP) = 0.0000833ms * 1 * 1200 = 100us */ 
  TIMER0->TCSR &= ~TIMER_TCSR_PRESCALE_Msk; // Set Prescale [0~255], must be set as 0 
  TIMER0->TCMPR = 1200;     // Set TCMPR [0~16777215] 
  
  /* Disable Timer0 */ 
  TIMER0->TCSR &= ~TIMER_TCSR_CEN_Msk; 
  
  /* Enable Timer0 interrupt and NVIC IRQ */ 
  TIMER0->TCSR |= TIMER_TCSR_IE_Msk;  // Enable Timer0 interrupt 
   NVIC_EnableIRQ(TMR0_IRQn);    // Enable Timer0 NVIC IRQ  
  NVIC_SetPriority(TMR0_IRQn, 0);  // Set Timer0 to 1st interrupt priority 
   
  /* Clear Timer0 interrupt counter */ 
  Timer0_int_cnt = 0; 
}    
 
//******************************* 
// Timer0 Delay (N x 100us) 
// 
//******************************* 
void Timer0_Delay(uint32_t N) 
{   
/* Reset Timer0 counter, prescale and disable Timer0 */ 
     TIMER0->TCSR |= TIMER_TCSR_CRST_Msk; 
    
     /* Reset Timer0 interrupt counter */ 
     Timer0_int_cnt = 0; 
 
     /* Enable Timer0 */ 
     TIMER0->TCSR |= TIMER_TCSR_CEN_Msk; 
 
     /* Wait msec */ 
     while(1){ 
      if(Timer0_int_cnt >= N) 
     break; 
     }                                                                                  
     
     /* Disable Timer0 */ 
    TIMER0->TCSR &= ~TIMER_TCSR_CEN_Msk; 
} 

//******************************* 
// Initial Timer1 for CEC Decoder 
// 
// Use HCLK as Timer1 clock source 
// Use Timer Capture Function 
// Timer1 is 24-bit resolution 
//******************************* 
void Init_Timer1(void) 
{ 
  /* Reset Timer1 counter, prescale and disable Timer1 (CEN = 0) */ 
  TIMER1->TCSR |= TIMER_TCSR_CRST_Msk; 
  
  /* Select Operation mode */ 
  TIMER1->TCSR &= ~TIMER_TCSR_MODE_Msk; 
  TIMER1->TCSR |= TIMER_ONESHOT_MODE;  // Select one-shot mode for operation mode and enable TDR function 
 
  /* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP) */ 
  TIMER1->TCSR &= ~TIMER_TCSR_PRESCALE_Msk; // Set Prescale [0~255], must be set as 0 
  TIMER1->TCMPR = 0xFFFFFF; 
 
  /* Set TM1_EXT at falling-edge interrupt and to reset counter */ 
  TIMER1->TEXCON &= ~TIMER_TEXCON_TEX_EDGE_Msk; 
  TIMER1->TEXCON |= TIMER_CAPTURE_COUNTER_RESET_MODE; 
 
  /* Enable TM1_EXT debounce function */ 
  TIMER1->TEXCON |= TIMER_TEXCON_TEXDB_Msk;  
 
  /* Enable TM1_EXT interrupt */ 
  TIMER1->TEXCON |= TIMER_TEXCON_TEXIEN_Msk; 
 
  /* Enable TM1_EXT pin */ 
  TIMER1->TEXCON |= TIMER_TEXCON_TEXEN_Msk; 
 
  /* Disable Timer1 */ 
  TIMER1->TCSR &= ~TIMER_TCSR_CEN_Msk;  
 
/* Enable Timer1 interrupt and NVIC IRQ */ 
  TIMER1->TCSR |= TIMER_TCSR_IE_Msk;  // Enable Timer1 interrupt 
     NVIC_EnableIRQ(TMR1_IRQn);       // Enable Timer1 NVIC IRQ 
  NVIC_SetPriority(TMR1_IRQn, 3);   // Set Timer1 to lowest interrupt priority  
  /* Reset CEC state */ 
  CEC_State = 0; 
} 
 
void GPAB_IRQHandler(void) 
{ 
   /* To check if PB.10 interrupt occurred */ 
  if(GPIO_GET_INT_FLAG(PB, BIT10)){ 
    Key_flag = 1; 
   GPIO_CLR_INT_FLAG(PB, BIT10); 
  }else{ 
   /* Un-expected interrupt. Just clear all PA, PB interrupts */ 
   PA->ISRC = PA->ISRC; 
   PB->ISRC = PB->ISRC; 
   printf("\nUn-expected interrupts."); 
  } 
} 

//******************************* 
// Timer0 call back function (Timer0 interrupt subroutine) 
// 
//******************************* 
void TMR0_IRQHandler(void)  
{  
  /* Clear TIMER0 Timeout Interrupt Flag */ 
  TIMER0->TISR = TIMER_TISR_TIF_Msk; 
  
  /* Timer0 interrupt counter + 1 */ 
  Timer0_int_cnt = Timer0_int_cnt + 1; 
} 
 
//******************************* 
// Timer1 call back function (Timer1 interrupt subroutine) 
// 
//******************************* 
void TMR1_IRQHandler(void)  
{  
  uint32_t TCAP1_tmp = 0; 
  
  /* Clear TIMER1 Capture Interrupt Flag */ 
   TIMER1->TEXISR = TIMER_TEXISR_TEXIF_Msk; 
  
  TMR1_flag = 1; 
  
  // CEC Bus is Low 
  if(PE5 == 0){ 
   // ACK for Header Block and Data Block 
   if(((CEC_State == 10) || (CEC_State == 20)) && (CEC_STB_Ready == 1)){ 
    if(CEC_State == 10) 
     RX_DATA[Rx_CEC_Data_Len] = CEC_HEAD; 
    else 
     RX_DATA[Rx_CEC_Data_Len] = CEC_DATA; 
    
    Rx_CEC_Data_Len++;    
          
    // Rx  
    if(Tx_flag == 0){ 
     if(CEC_Head_Dest_flag == 1){ 
      /* Set T1EX at falling-edge interrupt and to reset counter */ 
      TIMER1->TEXCON &= ~TIMER_TEXCON_TEX_EDGE_Msk;  
      TIMER1->TEXCON |= TIMER_CAPTURE_COUNTER_RESET_MODE; 
     
      /* ACK: PB.11 output low 1.5ms */ 
      PB11 = 0; 
      Timer0_Delay(CEC_BIT_0_L); 
      PB11 = 1; 
     } 
     
     // Next Data 
     if(((CEC_State == 10) && (CEC_EOM_H == 0)) || ((CEC_State == 20) && (CEC_EOM_D == 0))){ 
      CEC_State = 11; 
     // The End 
     }else{ 
      PB11 = 1; 
      CEC_State = 0; 
      CEC_STB_Ready = 0; 
      Rx_CEC_Data_flag = 1; 
     } 
    } 
    // Tx 
    else{ 
     /* Set T1EX at rising-edge interrupt and to capture counter */ 
     TIMER1->TEXCON |= TIMER_CAPTURE_RISING_EDGE; 
     TIMER1->TEXCON &= ~TIMER_TEXCON_RSTCAPSEL_Msk; 
    
     /* Enable Timer1 */ 
     TIMER1->TCSR |= TIMER_TCSR_CEN_Msk; 
    } 
   } 
   // The Other non-ACK Fields 
   else{ 
    /* Set T1EX at rising-edge interrupt and to capture counter */ 
    TIMER1->TEXCON |= TIMER_CAPTURE_RISING_EDGE;  
    TIMER1->TEXCON &= ~TIMER_TEXCON_RSTCAPSEL_Msk; 
    
    /* Enable Timer1 */ 
    TIMER1->TCSR |= TIMER_TCSR_CEN_Msk; 
   } 
  }  
  // CEC Bus is High  
  else{ 
   TCAP1_tmp = TIMER1->TCAP; 
   /* Set T1EX at falling-edge interrupt and to reset counter */ 
   TIMER1->TEXCON &= ~TIMER_TEXCON_TEX_EDGE_Msk;  
   TIMER1->TEXCON |= TIMER_TEXCON_RSTCAPSEL_Msk; 
     
   /* Disable Timer1 (CEN = 0) */ 
   TIMER1->TCSR &= ~TIMER_TCSR_CEN_Msk;  
        
   // Start Bit 
   if(CEC_State == 0){   
    CEC_State++; 
  
    if((TCAP1_tmp >= STB_MIN) && (TCAP1_tmp <= STB_MAX)){ // Start bit 
     CEC_STB_Ready = 1; // Set STart Bit Ready  
     CEC_HEAD = 0; 
     CEC_EOM_H = -1;  
     CEC_ACK_H = -1;  
     CEC_EOM_D = -1;  
     CEC_ACK_D = -1;   
     CEC_Head_Dest_flag = 0; 
     Rx_CEC_Data_flag = 0; 
     Rx_CEC_Data_Len = 0; 
    }else{ 
     CEC_State = 0; 
     CEC_STB_Ready = 0; 
    } 
   } 
   // Header Block 
   else if((CEC_State >= 1 && CEC_State < 9) && (CEC_STB_Ready == 1)){  
    CEC_State++; 
 
    CEC_HEAD = CEC_HEAD << 1; 
    if((TCAP1_tmp >= BIT_1_MIN) && (TCAP1_tmp <= BIT_1_MAX))    // 1         
			CEC_HEAD |= 0x01; 
    else if((TCAP1_tmp >= BIT_0_MIN) && (TCAP1_tmp <= BIT_0_MAX))  // 0 
     CEC_HEAD &= 0xFE;   
    else{ 
     CEC_State = 0; 
     CEC_STB_Ready = 0; 
    } 
   } 
   // End of Message for Header Block 
   else if((CEC_State == 9) && (CEC_STB_Ready == 1)){ 
    CEC_State++; 
 
    // Rx 
    if(Tx_flag == 0){  
     /* Check the Destination whether is belong to this device? */ 
     if((CEC_HEAD & 0x0F) == CEC_HEAD_DEST) 
      CEC_Head_Dest_flag = 1; 
    } 
     
    if((TCAP1_tmp >= BIT_1_MIN) && (TCAP1_tmp <= BIT_1_MAX))   // 1 
     CEC_EOM_H = 1; 
    else if((TCAP1_tmp >= BIT_0_MIN) && (TCAP1_tmp <= BIT_0_MAX)) // 0 
     CEC_EOM_H = 0;    
    else{ 
     CEC_State = 0; 
     CEC_STB_Ready = 0; 
    }  
   } 
   // ACK for Header Block 
   else if((CEC_State == 10) && (CEC_STB_Ready == 1)){    
    // Tx 
    if(Tx_flag == 1){ 
     if((TCAP1_tmp >= BIT_1_MIN) && (TCAP1_tmp <= BIT_1_MAX))      // 1      
			 CEC_ACK_H = 1; 
     else if((TCAP1_tmp >= BIT_0_MIN) && (TCAP1_tmp <= BIT_0_MAX))  // 0      
			 CEC_ACK_H = 0;  
     
     /* Receive Not ACK for Header Block */ 
     if(CEC_ACK_H != 0){ 
      PB11 = 1; 
      CEC_State = 0; 
      CEC_STB_Ready = 0; 
      Tx_NACK_H = 1; 
     }else 
      CEC_State = 11; 
    } 
   } 
   // Data Blcok 
   else if((CEC_State >= 11 && CEC_State < 19) && (CEC_STB_Ready == 1)){  
    CEC_State++; 
 
    CEC_DATA = CEC_DATA << 1; 
    if((TCAP1_tmp >= BIT_1_MIN) && (TCAP1_tmp <= BIT_1_MAX))   // 1 
     CEC_DATA |= 0x01; 
    else if((TCAP1_tmp >= BIT_0_MIN) && (TCAP1_tmp <= BIT_0_MAX)) // 0 
     CEC_DATA &= 0xFE;  
    else{ 
     CEC_State = 0; 
     CEC_STB_Ready = 0; 
    }     
   } 
   // End of Message for Data Block 
   else if((CEC_State == 19) && (CEC_STB_Ready == 1)){ 
    CEC_State++; 
 
    if((TCAP1_tmp >= BIT_1_MIN) && (TCAP1_tmp <= BIT_1_MAX))   // 1 
     CEC_EOM_D = 1; 
    else if((TCAP1_tmp >= BIT_0_MIN) && (TCAP1_tmp <= BIT_0_MAX)) // 0 
     CEC_EOM_D = 0;   
    else{ 
     CEC_State = 0; 
     CEC_STB_Ready = 0; 
    }   
   } 
   // ACK for Data Block 
   else if((CEC_State == 20) && (CEC_STB_Ready == 1)){ 
    // Tx 
    if(Tx_flag == 1){ 
     if((TCAP1_tmp >= BIT_1_MIN) && (TCAP1_tmp <= BIT_1_MAX))   // 1 
      CEC_ACK_D = 1; 
     else if((TCAP1_tmp >= BIT_0_MIN) && (TCAP1_tmp <= BIT_0_MAX)) // 0  
      CEC_ACK_D = 0;  
     
     /* Receive Not ACK for Data Block */ 
     if(CEC_ACK_D != 0){ 
      PB11 = 1; 
      CEC_State = 0; 
      CEC_STB_Ready = 0; 
      Tx_NACK_D = 1; 
     }else 
      CEC_State = 11; 
    } 
   } 
  } 
} 

 
//*******************************               
// Send CEC Format 
// 
// PortB.11 
//*******************************    
int8_t SendCEC(uint8_t data[], uint8_t byte_len) 
{ 
  uint8_t nbyte; 
  int8_t nbit; 
 
  Tx_flag = 1;  // 1=Transmit 
  Tx_NACK_H = 0; // clear Tx_NACK_H 
  Tx_NACK_D = 0; // clear Tx_NACK_D   
  
  CEC_Broadcast_flag = ((data[0] & 0x0f) == 0x0f); 
 
  /* Waitting for PB.11 (CEC BUS) to release High */ 
  while(PB11 == 0); 
  
  /* Check CEC Bus = High that over free time >= 5 * 2.4ms */ 
  TMR1_flag = 0; 
  // Delay free time >= 5 * 2.4ms 
  Timer0_Delay(CEC_FREE_TIME_NI);  
  if(TMR1_flag != 0){  
   printf("\nDetect CEC Bus = Low!!!");  
   // Turn back as a follower 
   Tx_flag = 0; 
   // Delay free time >= 3 * 2.4ms  
   Timer0_Delay(CEC_FREE_TIME_RS);  
   return -1;  
  } 
 
  /* Send out Start Bit */ 
  PB11 = 0; 
  Timer0_Delay(CEC_START_BIT_L); 
  PB11 = 1; 
  if(PB11 == 0){ 
   printf("\nArbitration Loss at Start Bit!!!"); 
   // Turn back as a follower 
   Tx_flag = 0; 
   // Delay free time >= 3 * 2.4ms 
   Timer0_Delay(CEC_FREE_TIME_RS);  
   return -1; 
  } 
  Timer0_Delay(CEC_START_BIT_H); 
    
  /* Send out Header Code and Data Code */ 
  for(nbyte=0; nbyte < byte_len; nbyte++){ 
   /* Send out Bits of Header Code and Data Code */  
   for(nbit=7; nbit >= 0; nbit--){ 
    if(data[nbyte] & (1 << nbit)){  // MSB first 
          PB11 = 0; 
     Timer0_Delay(CEC_BIT_1_L); 
     PB11 = 1; 
     Timer0_Delay(CEC_BIT_1_H); 
        }else{ 
           PB11 = 0; 
     Timer0_Delay(CEC_BIT_0_L); 
     PB11 = 1; 
     Timer0_Delay(CEC_BIT_0_H); 
    } 
    if((nbyte == 0) && (nbit >= 4)){ 
     if(((data[0] >> nbit) & 0x01) != (CEC_HEAD & 0x01) ){ 
      printf("\nArbitration Loss at Initiator Bits!!!"); 
      // Turn back as a follower 
      Tx_flag = 0; 
      // Delay free time >= 3 * 2.4ms  
      Timer0_Delay(CEC_FREE_TIME_RS);  
      return -1; 
     } 
    }  
   } 
  
   /* Send out EOM Bit */ 
   if(nbyte == byte_len - 1){ 
    PB11 = 0; 
    Timer0_Delay(CEC_BIT_1_L); 
    PB11 = 1; 
    Timer0_Delay(CEC_BIT_1_H); 
   }else{ 
    PB11 = 0; 
    Timer0_Delay(CEC_BIT_0_L); 
    PB11 = 1; 
    Timer0_Delay(CEC_BIT_0_H); 
   } 
 
   /* Send out ACK Bit = 1 */ 
   PB11 = 0; 
   Timer0_Delay(CEC_BIT_1_L); 
   PB11 = 1; 
   Timer0_Delay(CEC_BIT_1_H); 
 
   /* Check ACK for Header Block */ 
   if(nbyte == 0 && Tx_NACK_H == 1){ 
    if(CEC_Broadcast_flag == 0){ 
     printf("\nNo Found Destination Device!!!"); 
     // Turn back as a follower 
     Tx_flag = 0; 
     // Delay free time >= 3 * 2.4ms  
     Timer0_Delay(CEC_FREE_TIME_RS);   
     return -1; 
    } 
   }else if(nbyte != 0 && Tx_NACK_D == 1){ 
    if(CEC_Broadcast_flag == 0){ 
     printf("\nNo ACK for Data Block!!!"); 
     // Turn back as a follower 
     Tx_flag = 0; 
     // Delay free time >= 3 * 2.4ms  
     Timer0_Delay(CEC_FREE_TIME_RS);   
     return -1;  
    } 
   } 
  } 
 
  // Turn back as a follower 
  Tx_flag = 0; 
 
  // Delay free time >= 7 * 2.4ms for next time to transmit 
  Timer0_Delay(CEC_FREE_TIME_PI);  
 
  return 0; 
}   

//*******************************         
// Send CEC Command 
// 
//  
//*******************************   
void Send_CEC_Command(uint8_t data[], uint8_t byte_len) 
{ 
  uint8_t i; 
  
  for(Tx_Retry_Cnt = 0; Tx_Retry_Cnt < CEC_MAX_RETRY; Tx_Retry_Cnt++){  
   if(SendCEC(data, byte_len) == 0){  
    printf("\nTransmitted CEC Data: "); 
    for(i=0;i<byte_len;i++)   
     printf("%02x ", data[i]); 
    break; 
   }else{ 
    printf("\nRe-transmit!!! <%d>", Tx_Retry_Cnt);  
   } 
  } 
} 

//*******************************               
// Decode CEC Command 
// 
//  
//*******************************   

void Decode_CEC_Command(void) 
{ 
  Initiator_Device = (RX_DATA[0] & 0xf0) >> 4; 
  
  switch(RX_DATA[1]){ 
   case Give_Physical_Address: 
    Send_CEC_Command(REPORT_PHYSICAL_ADDRESS, 5); 
    break; 
   case Give_Device_Vendor_ID: 
    Send_CEC_Command(DEVICE_VENDOR_ID, 5); 
      break; 
   case Give_Device_Power_Status: 
    REPORT_POWER_STATUS[0] = Initiator_Device; 
    Send_CEC_Command(REPORT_POWER_STATUS, 3); 
    break; 
   case Get_CEC_Version: 
    CEC_VERSION[0] = Initiator_Device; 
    Send_CEC_Command(CEC_VERSION, 3); 
    break; 
   case Image_View_On: 
    PB13 = 0; // Turn on LED (as TV/Projector on) 
    break; 
         case Text_View_On: 
    PB13 = 0; // Turn on LED (as TV/Projector on) 
    break; 
   case Inactive_Source: 
    PB13 = 1; // Turn off LED (as TV/Projector off) 
    break; 
   default: 
    break; 
  } 
 } 

/*----------------------------------------------------------------------------------*/ 
/*  MAIN function                                                                   */ 
/*----------------------------------------------------------------------------------*/  
int main (void) 
{  
 uint8_t i; 
  
 /* Init System, IP clock and multi-function I/O */ 
 SYS_Init();      
 
 /* Init UART0 for printf */ 
 UART0_Init(); 
 
 printf("+----------------------------------------------+\n"); 
 printf("|              HDMI CEC Sample Code            |\n"); 
 printf("+----------------------------------------------+\n");  
 printf("\n"); 
 
 /* Use PB.13 to turn on/off LED */ 
 GPIO_SetMode(PB, BIT13, GPIO_PMD_OUTPUT); 
 PB13 = 1; 
  
 /* Configure PB.10 as Quasi-bidirectional mode and enable interrupt by falling edge trigger */ 
 GPIO_SetMode(PB, BIT10, GPIO_PMD_QUASI); 
 GPIO_EnableInt(PB, 10, GPIO_INT_FALLING);  
 NVIC_EnableIRQ(GPAB_IRQn); 
  
 /* Enable interrupt de-bounce function and select de-bounce sampling cycle time is 1024 clocks of LIRC clock */ 
   GPIO_SET_DEBOUNCE_TIME(GPIO_DBCLKSRC_LIRC, GPIO_DBCLKSEL_1024); 
  GPIO_ENABLE_DEBOUNCE(PB, BIT10); 
 
 /* Initial PB.11 as Open-drain mode and output high */ 
 GPIO_SetMode(PB, BIT11, GPIO_PMD_OPEN_DRAIN); 
 PB11 = 1; 
   
 /* set PE.5 as Input mode */ 
 GPIO_SetMode(PE, BIT5, GPIO_PMD_INPUT); 
 PE5 = 1; 
  
 /* Initial Timer0 for Delay function */      
 Init_Timer0(); 
 
    /* Initial Timer1 for Timer Capture function */  
 Init_Timer1(); 
    
 Tx_flag = 0; 
 Key_flag = 0; 
 Rx_CEC_Data_flag = 0; 
  
 while(1){ 
  if(Key_flag == 1){ 
   GPIO_DisableInt(PB, 10); 
   Key_flag = 0; 
   STANDBY[0] = Initiator_Device; // Device Standby, 0x0f: Broadcast Standby 
   Send_CEC_Command(STANDBY, 2); 
   GPIO_EnableInt(PB, 10, GPIO_INT_FALLING); 
  }else if(Rx_CEC_Data_flag == 1){ 
   Rx_CEC_Data_flag = 0; 
    
   if(CEC_Head_Dest_flag == 1){ 
    printf("\nReceived CEC Data: "); 
    for(i=0;i<Rx_CEC_Data_Len;i++)   
     printf("%02x ", RX_DATA[i]); 
    Decode_CEC_Command(); 
   }else{ 
    printf("\nMonitored CEC Data: "); 
    for(i=0;i<Rx_CEC_Data_Len;i++)   
     printf("%02x ", RX_DATA[i]); 
   } 
  } 
 } 
} 
 
